/****H* Ayu/Ayudame_lib.h
 *  NAME
 *    Ayudame_lib.h - intern header file for the Ayudame library package
 *  DESCRIPTION
 *    contains include directives for headers, system dependent macros and
 *    the version string macro.
 *    Moreover these global constants:
 *    * AYU_n_threads
 *    * AYU_max_threads
 *    and these function declarations:
 *    * AYU_initSocket
 *    * AYU_write
 *    * AYU_timestamp
 *    * AYU_read
 *    * AYU_receiveThreadFunction
 *    * AYU_initControl
 *    * AYU_pause
 *  COPYRIGHT
 *    (C) 2010-2015 HLRS, University of Stuttgart
 *    Ayudame is published under the terms of the BSD license.
 ******/

#ifndef AYUDAME_LIB_H
#define AYUDAME_LIB_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define AYU_VERSION PACKAGE_VERSION

#include <dlfcn.h>
#include <execinfo.h>

//#include <cxxabi.h>

//#define UNW_LOCAL_ONLY
//#include <libunwind.h>

#include <pthread.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <iostream>
#include <string>
#include <map>
#include <set>

#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>

// for AYU_timestamp
#include <time.h>
#include <sys/time.h>
#ifdef __MACH__
#include <mach/clock.h>
#include <mach/mach.h>
#endif

// includes and defines for time measurement with rdtsc
#if defined(__linux__)
#  include <endian.h>
#elif defined(__FreeBSD__) || defined(__NetBSD__)
#  include <sys/endian.h>
#elif defined(__OpenBSD__)
#  include <sys/types.h>
// # define be16toh(x) betoh16(x)
// # define be32toh(x) betoh32(x)
#  define be64toh(x) betoh64(x)
#elif defined(__APPLE__)
#include <libkern/OSByteOrder.h>
#define be64toh(x) OSSwapInt64(x)
#define htobe64(x) OSSwapInt64(x)
#else
#  error ERROR: System not recognised!
#endif

#ifndef htobe64
#  include <byteswap.h>
#  if __BYTE_ORDER == __LITTLE_ENDIAN
#    define htobe64(x) __bswap_64 (x)
#    define be64toh(x) __bswap_64 (x)
#  else
#    define htobe64(x) (x)
#    define be64toh(x) (x)
#  endif
#endif

/*
 * #ifndef htobe64(x) #define htobe64(x) ((((uint64_t)htonl(x)) << 32) +
 * htonl(x >> 32)) #define be64toh(x) ((((uint64_t)ntohl(x)) << 32) +
 * ntohl(x >> 32)) #endif
 */
// ---

// global variable
extern unsigned long AYU_n_threads;       // current number of threads
extern unsigned long AYU_max_threads;     // maximum number of threads

// global constant variables
const size_t AYU_buf_size = 8;          // size of message buffer
const size_t AYU_string_buf_size = 128; // size of string buffer

// forward declarations
// implemented in ayudame_lib.cc:
int AYU_initSocket (void);
int AYU_write (int c, void *buf, size_t buf_size);
bool is_verbose_greater_than(int verbosity_level);

extern "C"
{
  uint64_t AYU_timestamp (void);
}

// implemented in ayudame_control.cc:
int AYU_read(int c, void *buf, size_t buf_size);
void *AYU_receiveThreadFunction(void *c_);
int AYU_initControl (intptr_t c);
void AYU_pause(ayu_event_t event = AYU_EVENT_NULL, int taskId = 0);

#endif // AYUDAME_LIB_H
