# COPYRIGHT
#    (C) 2014-2015 HLRS, University of Stuttgart
#    Temanejo is published under the terms of the BSD license.
#
#    Other copyrights may apply

from __future__ import division
from __future__ import print_function


__debug = False


def half_avalanche1(a):
    # see ArjunShankar' response at http://stackoverflow.com/questions/11359441/hash-function-for-64-bit-to-10-bits
    # and http://burtleburtle.net/bob/hash/integer.html
    #if __debug: print("   a %s" % bin(a)  )
    a = (a+0x479ab41d) + (a<<8)
    #if __debug: print("   b %s" % bin(a)  )
    a = (a^0xe4aa10ce) ^ (a>>5)
    #if __debug: print("   c %s" % bin(a)  )
    a = (a+0x9942f0a6) - (a<<14)
    #if __debug: print("   d %s" % bin(a)  )
    a = (a^0x5aedd67d) ^ (a>>3)
    #if __debug: print("   e %s" % bin(a)  )
    a = (a+0x17bea992) + (a<<7)
    #if __debug: print("   f %s" % bin(a)  )
    return a;

def half_avalanche2(a):
    # see http://burtleburtle.net/bob/hash/integer.html
    #if __debug: print("   a %s" % bin(a)  )
    a = (a ^ 61) ^ (a >> 16)
    #if __debug: print("   b %s" % bin(a)  )
    a = a + (a << 3)
    #if __debug: print("   c %s" % bin(a)  )
    a = a ^ (a >> 4)
    #if __debug: print("   d %s" % bin(a)  )
    a = a * 0x27d4eb2d;
    #if __debug: print("   e %s" % bin(a)  )
    a = a ^ (a >> 15)
    #if __debug: print("   f %s" % bin(a)  )
    return a;


def hashto10bits(i):
    # see ArjunShankar' response at http://stackoverflow.com/questions/11359441/hash-function-for-64-bit-to-10-bits
    
    #if __debug: print("a %s" % bin(i)  )

    # Drop two LSBs ... actually we do not drop them as we have sequential ids, also
    #i >>= 2
    ##if __debug: print("b %s" % bin(i)  )
    
    # Get rid of the MSBs. Keep 46 bits.
    i &= 0x3fffffffffff
    #if __debug: print("c %s" % bin(i)  )
    
    # Get the 14 MSBs and fold them in to get a 32 bit integer.
    # The MSBs are mostly 0s anyway, so we don't lose much entropy.
    msbs = (i >> 32) << 18
    #if __debug: print("d msbs %s" % bin(i)  )
    i ^= msbs
    #if __debug: print("e %s" % bin(i)  )
    
    # Now we pass this through a decent 'half avalanche' hash function
    i = half_avalanche1(i);
    #if __debug: print("f %s" % bin(i)  )
    
    # For an 10-bit hash, use the 10 MSBs of the uint32_t returned.
    i = ((0b1111111111 << (32-10)) & i) >> (32-10)
    #if __debug: print("g %s" % bin(i)  )

    return i

if __name__ == "__main__":

    __debug = True

    collisions = {}


    for n in range (0,2*1024):
        h = hashto10bits(n)
        #if __debug: print(n, hashto10bits(n))
        if not h in collisions:
            collisions[h] = []
        collisions[h].append(n)


    #if __debug: print(collisions)

    #for h, ns in collisions.items():
    #    if len(ns)>1:
    #        if __debug: print("%s collisions for h = %s, %s" % (len(ns), h, ns))

    print("Efficiency of hash space usage: %.3f" % (len(collisions)/1024.))

