#****h* Temanejo/temanejo_client_config
# NAME
#   temanejo_client_config
#   - the client configuration module
# DESCRIPTION
#   temanejo_client_config.py implements a class containing the configuration
#   parameters that depend on the client (runtime) connected to Temanejo
# METHODS
#   init(client_id)
# AUTHOR
#   Steffen Brinkmann et al, HLRS <temanejo@hlrs.de>
# COPYRIGHT
#   (C) 2013-2016 HLRS, University of Stuttgart
#   temanejo_graph.py is published under the terms of the BSD license.
#******

# list of possible values of no_thread_id across different runtimes
candidates_no_thread_id = [
    -1,                # SMPSS, OmpSs, etc.  
    None ]             # default

# list of client ids

client_t = ['AYU_CLIENT_GENERIC',
            'AYU_CLIENT_CPPSS',
            'AYU_CLIENT_SMPSS',
            'AYU_CLIENT_OMPSS',
            'AYU_CLIENT_STARPU',
            'AYU_CLIENT_FASTFLOW',
            'AYU_CLIENT_MPI',
            'AYU_CLIENT_GOMP',
            'AYU_CLIENT_CILK',
            'AYU_CLIENT_DUMMY']

# module globals
runtime_type = None
waiton_func_id = None    # task id to interpret as wait_on or
                         # hidden task
no_task_id  = None       # task id when no task is assigned
no_func_id  = None       # func id when no task is assigned
no_thread_id  = None     # thread id when no thread is assigned

# default capabiliites for generic runtime
_default_break_preselecttask = True
_default_set_numthreads = False
_default_have_renaming = False
_default_have_blocktask = False
_default_have_makedeps = False
_default_have_priorizetask = False

# no connected yet
break_preselecttask = False
set_numthreads = False
have_renaming = False
have_blocktask = False
have_makedeps = False
have_priorizetask = False

# properties
nthreads = 0


def init(client_id):
    global runtime_type, waiton_func_id, no_task_id, no_func_id, \
        no_thread_id, break_preselecttask, set_numthreads, have_renaming, \
        have_blocktask, have_makedeps, have_priorizetask

    try:
        runtime_type = client_t[client_id]
    except IndexError:
        runtime_time = client_t[0]  # generic
    
    if client_id == client_t.index('AYU_CLIENT_CPPSS'):
        waiton_taskid = -1
        no_thread_id = -1
        no_task_id = -1
        no_func_id = -1
    elif client_id == client_t.index('AYU_CLIENT_SMPSS'):
        waiton_func_id = -1
        no_thread_id = -1
        no_task_id = -1
        no_func_id = -1
        break_preselecttask = True
        set_numthreads = True
        have_renaming = True
        have_blocktask = True
        have_makedeps = True
        have_priorizetask = True
    elif client_id == client_t.index('AYU_CLIENT_OMPSS'):
        waiton_func_id = -1
        no_thread_id = -1
        no_task_id = -1
        no_func_id = -1
        break_preselecttask = False
        set_numthreads = False
        have_renaming = False
        have_blocktask = True
        have_makedeps = True
        have_priorizetask = False
    elif client_id == client_t.index('AYU_CLIENT_STARPU'):
        waiton_func_id = -1
        no_thread_id = -1
        no_task_id = -1
        no_func_id = -1
    elif client_id == client_t.index('AYU_CLIENT_FASTFLOW'):
        waiton_func_id = -1
        no_thread_id = -1
        no_task_id = -1
        no_func_id = -1
    else:
        waiton_func_id = -1
        no_thread_id = -1
        no_task_id = -1
        no_func_id = -1
        break_preselecttask = _default_break_preselecttask
        set_numthreads = _default_set_numthreads
        have_renaming = _default_have_renaming
        have_blocktask = _default_have_blocktask
        have_makedeps = _default_have_makedeps
        have_priorizetask = _default_have_priorizetask

        # consistency check:
        if not no_thread_id in candidates_no_thread_id:
            import inspect
            print("** ERROR: unexpected no_thread_id in %s \n" \
                  "**    context: %s\n" \
                  "**    Please report to temanejo@hlrs.de.\n" % \
                  (inspect.currentframe().f_code.co_name, no_thread_id) )
            
